#include <gba.h>

typedef unsigned short u16;

//macro to change the video mode
#define SetMode(mode) REG_DISPCNT = (mode)

//define some video addresses
#define REG_DISPCNT *(volatile unsigned short*)0x4000000

#define BGPaletteMem ((unsigned short*)0x5000000)

#define REG_VCOUNT *(volatile unsigned short*)0x4000006

#define REG_DISPSTAT *(volatile unsigned short *)0x4000004

//define object attribute memory state address
#define SpriteMem ((unsigned short*)0x7000000)

//define object attribute memory image address
#define SpriteData ((unsigned short*)0x6010000)

//define object attribute memory palette address
#define SpritePal ((unsigned short*)0x5000200)

//misc sprite constants
#define OBJ_MAP_2D 0x0
#define OBJ_MAP_1D 0x40
#define OBJ_ENABLE 0x1000

//attribute0 stuff
#define ROTATION_FLAG 0x100
#define SIZE_DOUBLE 0x200
#define MODE_NORMAL 0x0
#define MODE_TRANSPARENT 0x400
#define MODE_WINDOWED 0x800
#define MOSAIC 0x1000
#define COLOR_16 0x0000
#define COLOR_256 0x2000
#define SQUARE 0x0
#define TALL 0x4000
#define WIDE 0x8000

//attribute1 stuff
#define HORIZONTAL_FLIP 0x1000
#define VERTICAL_FLIP 0x2000
#define SIZE_8 0x0
#define SIZE_16 0x4000
#define SIZE_32 0x8000
#define SIZE_64 0xC000

//attribute2 stuff
#define PRIORITY(n) ((n) << 10)
#define PALETTE(n) ((n) << 12)

//sprite structs
typedef struct tagSprite
{
unsigned short attribute0;
unsigned short attribute1;
unsigned short attribute2;
unsigned short attribute3;
}Sprite,*pSprite;


char square[]={        
		       1,1,1,1,1,1,1,1,
                       1,1,1,1,1,1,1,1,
                       1,1,1,1,1,1,1,1,
                       1,1,1,1,1,1,1,1,
                       1,1,1,1,1,1,1,1,
                       1,1,1,1,1,1,1,1,
                       1,1,1,1,1,1,1,1,
                       1,1,1,1,1,1,1,1,
                    };

//create an array of 128 sprites equal to OAM
Sprite sprites[128];

//function prototypes
void WaitForVsync(void);
void UpdateSpriteMemory(void);


int main(void) {
	signed short x = 50, y = 50;
	int char_number = 0;
	int n;

	//set the video mode--mode 2 with sprites
	SetMode(2 | OBJ_ENABLE | OBJ_MAP_1D);


	//move all sprites offscreen to hide them
	for(n = 0; n < 128; n++)
	{
		sprites[n].attribute0 = 160;
		sprites[n].attribute1 = 240;
	}

	

	for(n = 0; n < 64; n++){
		SpritePal[n] = RGB8(255,255,255);
	}

	
	
	//copy the sprite image into memory
	for(n = 0; n < 32; n++) {
		 SpriteData[n] =  ((short*)square)[n];
	}

	//setup the first sprite
	sprites[0].attribute0 = COLOR_256 | y;
	sprites[0].attribute1 = SIZE_8 | x;
	sprites[0].attribute2 = char_number;
	
	while(1)
	{
		//update sprite x position
		//update sprite attributes with new x,y position
		sprites[0].attribute0 = COLOR_256 | y;
		sprites[0].attribute1 = SIZE_8 | x;
		

		//wait for vertical retrace
		WaitForVsync();

		//display the sprite
		UpdateSpriteMemory();
	}
}


void WaitForVsync(void)
{
	while((REG_DISPSTAT & 1));
}

void UpdateSpriteMemory(void)
{
	int n;
	unsigned short* temp;
	temp = (unsigned short*)sprites;
	for(n = 0; n < 128*4; n++)
		SpriteMem[n] = temp[n];
	
}